"""Run ezCharts demo."""
import argparse

from dominate.tags import p
import numpy as np
import pandas as pd
from pkg_resources import resource_filename

import ezcharts as ezc
from ezcharts import util
from ezcharts.components.ezchart import EZChart
from ezcharts.components.fastcat import SeqSummary
from ezcharts.components.nextclade import NextClade, NXTComponent
from ezcharts.components.reports.labs import LabsReport
from ezcharts.components.theme import LAB_head_resources
from ezcharts.layout.snippets import Grid
from ezcharts.layout.snippets import Stats
from ezcharts.layout.snippets import Tabs
from ezcharts.layout.snippets.cards import Cards, ICard
from ezcharts.layout.snippets.progress import Progress
from ezcharts.plots import Plot
from ezcharts.plots.ideogram import ideogram

# Setup simple globals
WORKFLOW_NAME = 'wf-template'
REPORT_TITLE = f'{WORKFLOW_NAME}-report'


def main(args):
    """Demo for ezCharts functionality."""
    logger = util.get_named_logger("ezChrtDemo")

    # Good location to prepare plots
    logger.info('Building plots')

    def example_plot(style="line") -> Plot:
        """Create example plot."""
        df = pd.DataFrame({
            # as laid out in echarts docs for a dataset
            'product': [
                'Matcha Latte', 'Milk Tea', 'Cheese Cocoa', 'Walnut Brownie'],
            '2015': [43.3, 83.1, 86.4, 72.4],
            '2016': [85.8, 73.4, 65.2, 53.9],
            '2017': [93.7, 55.1, 82.5, 39.1]
        })
        # how we'd likely normally have it
        df = df.melt(
            id_vars=['product'], value_vars=['2015', '2016', '2017'],
            var_name='year', value_name='sales')
        if style == "line":
            plot = ezc.lineplot(data=df, x='year', y='sales', hue='product')
        elif style == "scatter":
            plot = ezc.scatterplot(data=df, x='year', y='sales', hue='product')
        elif style == 'histogram':
            plot = ezc.histplot(
                data=np.random.randint(0, 11, size=1000), bins=10)
        elif style == "bar":
            plot = ezc.barplot(data=df, x='year', y='sales', hue='product')
        else:
            raise ValueError("Unknown plot style")
        plot.title = {"text": f"Example {style} chart"}
        return plot

    # Then, time to construct a report
    logger.info('Building report')

    # Example data
    params = resource_filename('ezcharts', "data/test/params.json")
    versions = resource_filename('ezcharts', "data/test/versions.txt")
    nxt_json = resource_filename('ezcharts', "data/test/nextclade.json")
    fastcat_results = resource_filename(
        'ezcharts', "data/test/fastcat.stats.gz")

    # Create report
    # Note we need to add nextclade as a resource
    report = LabsReport(
        REPORT_TITLE, WORKFLOW_NAME, params, versions,
        head_resources=[*LAB_head_resources, NXTComponent])

    # Add a header badge via a method on the report element.
    report.add_badge('Test badge', bg_class="bg-primary")

    # Make a progress bar for the median accuracy box
    median_accuracy = Progress(
            value_min=0,
            value_max=100,
            value_now=98.9,
            bar_cls="progress-bar-striped",
            height=50)

    # Add something directly to main_content.
    with report.main_content:
        # Stats is a snippet, but we could also just add any html
        # tags here. Snippets are little reusable portions of html,
        # styles and or scripts in python class format. They often
        # expose methods for adding content to them in a simple way.
        # E.g.
        Stats(
            columns=3,
            items=[
                ('1213986', 'Read count'),
                (median_accuracy, 'Median accuracy'),
                ('10213 bp', 'Some other stat')
            ])

    # Add sequence summary component
    with report.add_section('Sequence summaries', 'Summaries'):
        SeqSummary(fastcat_results)

    # This also adds to main_content, but provides a nice
    # container snippet as a starting context.
    with report.add_section('Controls', 'Controls'):
        # This is an example of how to use bootstrap cards
        Cards(
            columns=2,
            items=[
                ICard(
                    head="Positive",
                    body="The positive passed",
                    footer="All postive controls within threshold",
                    classes="bg-success text-white"),
                ICard(
                    head="No Template Control",
                    body="The NTC failed",
                    footer="NTC outside set thresholds.",
                    classes="bg-danger text-white")])

    # This also adds to main_content, but provides a nice
    # container snippet as a starting context.
    with report.add_section('Alignment results', 'Results'):
        # This is the tabbed section with ezcharts!
        tabs = Tabs()
        with tabs.add_tab('Summary', True):
            # Grids are snippets that provide responsive
            # layouts via css grid
            with Grid():
                EZChart(example_plot("line"), 'epi2melabs')
                EZChart(example_plot("scatter"), 'epi2melabs')
                EZChart(example_plot("bar"), 'epi2melabs')
                EZChart(example_plot("histogram"), 'epi2melabs')
        with tabs.add_tab('Accuracy', False):
            p("This is a mixed tab!")
            EZChart(example_plot(), 'epi2melabs')
        with tabs.add_tab('Depth', False):
            p('Testing, testing, 1 2 3')

        # Dropdowns are nested one more level
        with tabs.add_dropdown_menu('Example'):
            with tabs.add_dropdown_tab('First', False):
                EZChart(example_plot(), 'epi2melabs')

    with report.add_section('Nextclade results', 'Nextclade', True):
        NextClade(nxt_json)

    with report.add_section('Human Genome', 'Genome'):
        EZChart(
            ideogram(blocks='cytobands'),
            'epi2melabs', height="800px")

    logger.info('Reticulating splines')
    report.write(args.output)


def argparser():
    """Argument parser for entrypoint."""
    parser = argparse.ArgumentParser(
        "ezcharts demo",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        add_help=False)
    parser.add_argument(
        "--output", default="ezcharts_demo_report.html",
        help="Output HTML file.")
    return parser



"""Get default tabs layouts."""
from dominate.tags import button, div, li, style, ul
from dominate.util import raw

from ezcharts.layout.base import IClasses, IStyles, Snippet
from ezcharts.layout.util import cls, css, render_template


class ITabsClasses(IClasses):
    """Tabs html classes."""

    container: str = cls("justify-content-center")
    tab_buttons_list: str = cls("nav", "nav-tabs", "mb-2")
    tab_buttons_list_item: str = cls("nav-item")
    tab_button: str = cls("nav-link", "px-0", "me-4", "text-muted")
    tab_contents_container: str = cls("tab-content", "p-5")
    tab_content: str = cls("tab-pane", "fade", "show")


class ITabsStyles(IStyles):
    """Tabs inline css styles."""

    tab_button: str = css(
        "margin-bottom: -1px",
        "font-weight: 600",
        "cursor: pointer",
        "border-color: transparent")
    tab_button_active: str = css(
        "border-bottom: 2px solid #0079a4",
        "color: #0079a4!important")


class Tabs(Snippet):
    """A styled div tag for creating tabbed layouts."""

    TAG = 'div'
    BUTTON_TEMPLATE = "#{{ id }}>ul>li>button { {{ styles }} };"
    BUTTON_TEMPLATE_ACTIVE = "#{{ id }}>ul>li>button.active { {{ astyles }} };"

    def __init__(
        self,
        styles: ITabsStyles = ITabsStyles(),
        classes: ITabsClasses = ITabsClasses()
    ) -> None:
        """Create tabs component."""
        super().__init__(
            styles=styles,
            classes=classes,
            className=classes.container)

        with self:
            style(raw(render_template(
                self.BUTTON_TEMPLATE,
                id=self.uid,
                styles=self.styles.tab_button)))
            style(raw(render_template(
                self.BUTTON_TEMPLATE_ACTIVE,
                id=self.uid,
                astyles=self.styles.tab_button_active)))
            self.buttons = ul(
                className=classes.tab_buttons_list,
                id="pills-tab",
                role="tablist")
            self.contents = div(
                className=classes.tab_contents_container,
                id="pills-tabContent")

    def add_tab(self, title: str, active: bool) -> div:
        """Add a tab button and content container."""
        _active = 'active' if active else ''
        lowered = title.lower()

        with self.buttons:
            with li(
                className=self.classes.tab_buttons_list_item,
                role="presentation"
            ):
                button(
                    title,
                    className=f"{self.classes.tab_button} {_active}",
                    id=f"{self.uid}-tabs-{lowered}-tab",
                    type="button",
                    role="tab",
                    **self._get_button_data_aria(lowered))

        with self.contents:
            return self._build_container(
                lowered, f"{self.classes.tab_content} {_active}")

    def add_dropdown_menu(self, title: str) -> ul:
        """Get a dropdown menu."""
        dropdown_link_data_aria = {"data-bs-toggle": "dropdown"}
        with self.buttons:
            with li(
                className=self.classes.tab_buttons_list_item + ' dropdown',
                role="presentation"
            ):
                button(
                    title,
                    className=f"{self.classes.tab_button} dropdown-toggle",
                    href="#",
                    **dropdown_link_data_aria)
                return ul(className="dropdown-menu")

    def add_dropdown_tab(self, title: str, active: bool) -> div:
        """Get a dropdown tab."""
        _active = 'active' if active else ''
        lowered = title.lower()

        with li():
            button(
                title,
                role="tab",
                className="dropdown-item",
                type="button",
                id=f"#{self.uid}-tabs-{lowered}-tab",
                **self._get_button_data_aria(lowered))

        with self.contents:
            return self._build_container(
                lowered, f"{self.classes.tab_content} {_active}")

    def _build_container(self, title: str, classes: str) -> div:
        """Get tab content container."""
        return div(
            className=classes,
            id=f"{self.uid}-tabs-{title}",
            role="tabpanel",
            **self._get_content_data_aria(title))

    def _get_button_data_aria(self, title: str):
        """Get ARIA data for tab buttons."""
        return {
            "data-bs-toggle": "tab",
            "data-bs-target": f"#{self.uid}-tabs-{title}",
            "aria-controls": f"{self.uid}-tabs-{title}",
        }

    def _get_content_data_aria(self, title: str):
        """Get ARIA data for tab content containers."""
        return {
            "aria-labelledby": f"{self.uid}-tabs-{title}-tab"
        }



"""Get default table layouts."""
from typing import List, Optional, Union

from dominate.tags import script, table, tbody, td, th, thead, tr
from dominate.util import raw

from ezcharts.layout.base import IClasses, Snippet
from ezcharts.layout.util import cls, render_template


class ITableClasses(IClasses):
    """Table html classes."""

    container: str = cls("table-responsive")
    table: str = cls("table", "table-striped", "table-hover")


class DataTable(Snippet):
    """A styled datatable wrapped in a div."""

    TAG = 'div'

    def __init__(
        self,
        headers: List[str],
        page_length: Optional[int] = 10,
        paging: Optional[bool] = True,
        classes: ITableClasses = ITableClasses(),
    ) -> None:
        """Create table."""
        super().__init__(
            styles=None,
            classes=classes,
            className=classes.container)

        with self:
            with table(
                id=self.uid + '_inner',
                className=self.classes.table
            ):
                self.head = thead()
                with self.head:
                    with tr():
                        for _header in headers:
                            th(_header, scope="col")
                self.body = tbody()
            script(render_template(
                """
                new simpleDatatables.DataTable('#{{ id }}_inner', {
                    searchable: true,
                    pageLength: {{ page_length }},
                    paging: {{ paging }},
                    columns: [
                        { select: 2, sort: 'desc' },
                    ]
                })
                """,
                id=self.uid,
                paging=str(paging).lower(),
                page_length=page_length))

    def add_row(
        self,
        title: Optional[str],
        columns: List[Union[str, int, float]]
    ) -> tr:
        """Add a row of cells to the table."""
        with self.body:
            with tr() as row:
                if title:
                    th(title, scope="row1")
                for column in columns:
                    td(raw(str(column)))
        return row
